@@ -51,6 +51,7 @@ from .util import CompleteChoice
 from .util import CustomHelpColorsGroup
 from .util import EnvVars
 from .util import add_history
+from .util import clean_string
 from .util import color_option
 from .util import database_callback
 from .util import database_exists_option
@@ -107,6 +108,13 @@ class ExportOutputChoice(CompleteChoice):
     ]
 
 
+def serializer(obj: object) -> object:
+    if isinstance(obj, Iterable):
+        return list(obj)
+    else:
+        return str(obj)
+
+
 def column_callback(_ctx: Context, _param: Option, value: tuple[str]) -> list[tuple[str, int]]:
     return [((vs := v.split(",", 1))[0], int(vs[1])) if "," in v else (v, 0) for v in map(str.strip, value) if v]
 
@@ -120,10 +128,6 @@ def id_callback(ctx: Context, param: Argument, value: tuple[str, ...]) -> tuple[
     return tuple(map(int, value))
 
 
-def clean_string(string: str, *, replacer: str = "□") -> str:
-    return "".join(c if 32 <= ord(c) <= 255 else replacer for c in string)
-
-
 def format_value(value: Any) -> str:
     return " ".join(value) if isinstance(value, (list, set)) else str(value)
 
@@ -145,7 +149,7 @@ def get_table(db: Database, table: str) -> Table:
 
 def print_table(ctx: Context, results: Cursor, headers: list[tuple[str, int]], ignore_width: bool) -> int:
     results_total: int = 0
-    if ctx.color:
+    if ctx.color is not False:
         terminal_width: int | None = None if ignore_width else get_terminal_size((0, 0)).columns or None
         terminal_width_total: int = terminal_width - (3 * (len(headers) - 1)) if terminal_width else None
         widths: list[int] = [w for _, w in headers]
@@ -185,7 +189,7 @@ def print_table(ctx: Context, results: Cursor, headers: list[tuple[str, int]], i
 def print_csv(results: Cursor, file: TextIO, delimiter: str) -> int:
     results_total: int = 0
     writer = csv_writer(file, delimiter=delimiter)
-    writer.writerow(results.columns)
+    writer.writerow([c.name for c in results.columns])
     for row in results.entries:
         results_total += 1
         writer.writerow(map(format_value, row.values()))
@@ -197,10 +201,10 @@ def print_json(results: Cursor, file: TextIO) -> int:
     file.write("[")
     if first := next(results.entries, None):
         results_total += 1
-        file.write(dumps(first, separators=(",", ":")))
+        file.write(dumps(first, default=serializer, separators=(",", ":")))
     for entry in results.entries:
         results_total += 1
-        file.write("," + dumps(entry, separators=(",", ":")))
+        file.write("," + dumps(entry, default=serializer, separators=(",", ":")))
     file.write("]")
     return results_total
 
