@@ -1,4 +1,5 @@
 from enum import Enum
+from enum import EnumMeta
 from enum import auto
 from operator import itemgetter
 from shutil import get_terminal_size
@@ -10,7 +11,7 @@ from typing import TypeVar
 from click import echo
 from faapi import FAAPI
 from faapi import SubmissionPartial
-from faapi.exceptions import DisabledAccount
+from faapi.exceptions import DisabledAccount, NotFound
 from faapi.exceptions import NoticeMessage
 from faapi.exceptions import ServerError
 from falocalrepo_database import Column
@@ -22,6 +23,7 @@ from requests import RequestException
 from requests import Response
 
 from .console.colors import *
+from .console.util import clean_string
 
 _FolderDownloader = Callable[[str, int | str], tuple[list[SubmissionPartial], str | int]]
 
@@ -40,6 +42,10 @@ class Folder(str, Enum):
     journals = "journals"
     userpage = "userpage"
 
+    @classmethod
+    def as_list(cls: EnumMeta) -> list[str]:
+        return [c.value for c in cls]
+
 
 def terminal_width() -> int:
     return get_terminal_size((0, 0)).columns
@@ -54,10 +60,6 @@ def format_entry(entry: dict[str, Any], columns: list[Column]) -> dict:
     return {k.upper().replace("_", ""): v for k, v in entry.items() if k.upper().replace("_", "") in columns_}
 
 
-def clean_string(string: str, *, replacer: str = "□") -> str:
-    return "".join(c if 32 <= ord(c) <= 255 else replacer for c in string)
-
-
 def get_downloader(api: FAAPI, folder: Folder) -> _FolderDownloader:
     if folder == Folder.gallery:
         return api.gallery
@@ -69,6 +71,26 @@ def get_downloader(api: FAAPI, folder: Folder) -> _FolderDownloader:
         raise KeyError(f"Unknown folder {folder}")
 
 
+def download_catch(func: Callable[..., T], *args, **kwargs) -> tuple[T | None, int]:
+    """
+    0 no errors
+
+    1 not found
+
+    2 user disabled
+
+    3 server error
+    """
+    try:
+        return func(*args, **kwargs), 0
+    except NotFound:
+        return None, 1
+    except DisabledAccount:
+        return None, 2
+    except (NoticeMessage, ServerError):
+        return None, 3
+
+
 class Bar:
     def __init__(self, length: int = 0, *, message: str = ""):
         self.length: int = length
@@ -122,6 +144,7 @@ class Downloader:
         self.downloaded: int = 0
         self.modified: int = 0
         self.user_errors: int = 0
+        self.user_deactivated: int = 0
         self.submission_errors: int = 0
         self.file_errors: int = 0
         self.thumbnail_errors: int = 0
@@ -133,6 +156,8 @@ class Downloader:
             items.append(("Downloaded", self.downloaded))
         if self.modified:
             items.append(("Modified", self.modified))
+        if self.user_deactivated:
+            items.append(("Users deactivated", self.user_errors))
         if self.user_errors:
             items.append(("User errors", self.user_errors))
         if self.submission_errors:
@@ -200,41 +225,12 @@ class Downloader:
         except RequestException:
             return None
 
-    def download_catch(self, func: Callable[..., T], *args, **kwargs) -> tuple[T | None, int]:
-        """
-        0 no errors
-
-        1 not found
-
-        2 user not found
-
-        3 user disabled
-
-        4 server error
-        """
-        try:
-            return func(*args, **kwargs), 0
-        except (NoticeMessage, ServerError) as err:
-            err_args: str = "".join(err.args).lower()
-            if "not in our database" in err_args:
-                return None, 1
-            elif "cannot be found" in err_args or \
-                    "could not be found" in err_args or \
-                    "user not found" in err_args:
-                return None, 2
-            else:
-                self.bar_message("SERVER ERR", red)
-                self.bar_close()
-                return None, 4
-        except DisabledAccount:
-            return None, 3
-
     def err_to_bar(self, err: int, *, close: bool = True, close_end: str = "\n") -> int:
-        if err in (1, 2):
+        if err == 1:
             self.bar_message("NOT FOUND", red)
-        elif err == 3:
+        elif err == 2:
             self.bar_message("NOT ACTIVE", red)
-        elif err == 4:
+        elif err == 3:
             self.bar_message("SERVER ERR", red)
         if err and close:
             self.bar_close(close_end)
@@ -244,7 +240,7 @@ class Downloader:
                             thumbnail: str, replace: bool = False) -> int:
         self.bar_clear()
         self.bar_message("DOWNLOAD")
-        result, err = self.download_catch(self.api.submission, submission_id)
+        result, err = download_catch(self.api.submission, submission_id)
         if self.err_to_bar(err):
             self.submission_errors += 1
             return err
@@ -282,7 +278,7 @@ class Downloader:
                  nl=self.output == OutputType.simple, color=self.color)
             self.bar()
             self.bar_message("DOWNLOAD")
-            result, err = self.download_catch(self.api.journals, user, page)
+            result, err = download_catch(self.api.journals, user, page)
             if err:
                 self.user_errors += 1
                 self.err_to_bar(err)
@@ -299,7 +295,14 @@ class Downloader:
                 self.bar_message("SEARCHING")
                 if journal.id in self.db.journals:
                     self.bar_message("IN DB", green)
-                    if not self.dry_run and self.db.journals.set_user_update(journal.id, 1):
+                    if self.dry_run:
+                        stop -= 1
+                        if clear_last_found:
+                            self.bar_close("")
+                            self.clear_line()
+                        else:
+                            self.bar_close()
+                    elif self.db.journals.set_user_update(journal.id, 1):
                         self.db.commit()
                         self.bar_message("UPDATED", green)
                         self.modified += 1
@@ -342,7 +345,7 @@ class Downloader:
                  nl=self.output == OutputType.simple, color=self.color)
             self.bar()
             self.bar_message("DOWNLOAD")
-            result, err = self.download_catch(downloader, user, next_page)
+            result, err = download_catch(downloader, user, next_page)
             if err:
                 self.user_errors += 1
                 self.err_to_bar(err)
@@ -359,21 +362,26 @@ class Downloader:
                 self.bar_message("SEARCHING")
                 if sub_partial.id in self.db.submissions:
                     self.bar_message("IN DB", green)
-                    if not self.dry_run and folder != Folder.favorites and \
-                            self.db.submissions.set_user_update(sub_partial.id, 1):
+                    if self.dry_run:
+                        stop -= 1
+                        if clear_last_found and stop == 0:
+                            self.bar_close("")
+                            self.clear_line()
+                        else:
+                            self.bar_close()
+                    elif folder != Folder.favorites and self.db.submissions.set_user_update(sub_partial.id, 1):
                         self.db.commit()
                         self.bar_message("UPDATED", green)
                         self.modified += 1
                         self.bar_close()
-                    elif not self.dry_run and folder == Folder.favorites and \
-                            self.db.submissions.add_favorite(sub_partial.id, user):
+                    elif folder == Folder.favorites and self.db.submissions.add_favorite(sub_partial.id, user):
                         self.db.commit()
                         self.bar_message("ADDED FAV", green)
                         self.modified += 1
                         self.bar_close()
                     else:
                         stop -= 1
-                        if clear_last_found:
+                        if clear_last_found and stop == 0:
                             self.bar_close("")
                             self.clear_line()
                         else:
@@ -383,7 +391,7 @@ class Downloader:
                     self.bar_close()
                 else:
                     self.download_submission(sub_partial.id, int(folder != Folder.favorites),
-                                             user if folder == Folder.favorites else None, sub_partial.thumbnail_url)
+                                             [user] if folder == Folder.favorites else None, sub_partial.thumbnail_url)
                 if stop == 0:
                     return 0
             self.clear_line()
@@ -398,12 +406,12 @@ class Downloader:
             self.clear_line()
             return 0
         self.bar_message("DOWNLOAD")
-        user, err = self.download_catch(self.api.user, username)
+        user, err = download_catch(self.api.user, username)
         self.err_to_bar(err)
         if err:
             self.user_errors += 1
             return err
-        added: bool = (current := self.db.users[username][UsersColumns.USERPAGE.value.name].strip()) == ""
+        added: bool = (current := self.db.users[username][UsersColumns.USERPAGE.value.name]) == ""
         updated: bool = user.profile != current
         if not added and not updated:
             self.bar_message("IN DB")
@@ -437,13 +445,14 @@ class Downloader:
                     err = self.download_user_journals(user, stop, stop > 0)
                 else:
                     err = self.download_user_submissions_folder(user, Folder[folder.lower()], stop, stop > 0)
-                if err in (1, 2, 3):
+                if err in (1, 2):
                     if self.dry_run:
                         pass
                     elif user_added:
                         del self.db.users[user]
                     else:
                         self.db.users.deactivate(user)
+                        self.user_deactivated += 1
                     self.db.commit()
                     break
                 self.bar_close()
@@ -453,16 +462,26 @@ class Downloader:
         self._download_users([(u, folders) for u in users])
 
     def download_users_update(self, users: list[str], folders: list[str], stop: int, deactivated: bool):
-        users_cursor = self.db.users[users] if users else self.db.users.select(order=[UsersColumns.USERNAME.value.name])
+        users_cursor: Iterable[dict] = self.db.users[users] if users else self.db.users.select(
+            order=[UsersColumns.USERNAME.value.name])
         users_folders: Iterable[tuple[str, list[str]]]
         users_folders = ((u[UsersColumns.USERNAME.value.name],
-                          [f for f in sorted(u[UsersColumns.FOLDERS.value.name])])
+                          [f for f in u[UsersColumns.FOLDERS.value.name]])
                          for u in users_cursor)
+
+        users_folders = sorted(users_folders, key=lambda uf: users.index(uf[0]) if users else uf[0])
+
         if deactivated:
             users_folders = ((u, [*map(lambda f: f.strip("!"), fs)]) for u, fs in users_folders)
         else:
             users_folders = ((u, fs) for u, fs in users_folders if not any("!" in f for f in fs))
-        users_folders = ((u, [f for f in fs if f in folders or not folders]) for u, fs in users_folders)
+
+        if folders:
+            users_folders = ((u, sorted(filter(folders.__contains__, fs), key=folders.index))
+                             for u, fs in users_folders)
+        else:
+            users_folders = ((u, sorted(fs, key=Folder.as_list().index)) for u, fs in users_folders)
+
         self._download_users(users_folders, stop)
 
     # noinspection DuplicatedCode
@@ -502,7 +521,7 @@ class Downloader:
                 self.bar_message("SKIPPED", green)
                 self.bar_close()
                 continue
-            journal, err = self.download_catch(self.api.journal, journal_id)
+            journal, err = download_catch(self.api.journal, journal_id)
             if self.err_to_bar(err):
                 continue
             self.db.journals.save_journal({
